# coding: utf-8

from django.conf.urls import url
from electron_pdf.views import PDFTemplateView

urlpatterns = [
    url(r'^test/$', PDFTemplateView.as_view(
        template_name='test.html',
        filename='test.pdf',
        show_content_in_browser=True,
        get_context_data=lambda: {'company_name': 'Namespace OÜ'},
    ), name='pdf'),
]



# coding: utf-8

from __future__ import absolute_import

from django.http import HttpResponse
from django.template.response import TemplateResponse
from django.views.generic import TemplateView

from electron_pdf.utils import content_disposition_filename, render_pdf_from_template


class PDFResponse(HttpResponse):
    """HttpResponse that sets the headers for PDF output."""

    def __init__(self, content, status=200, content_type=None, filename=None,
                 show_content_in_browser=None, *args, **kwargs):

        if content_type is None:
            content_type = 'application/pdf'

        super(PDFResponse, self).__init__(content=content,
                                          status=status,
                                          content_type=content_type)
        self.set_filename(filename, show_content_in_browser)

    def set_filename(self, filename, show_content_in_browser):
        self.filename = filename
        if filename:
            fileheader = 'attachment; filename={0}'
            if show_content_in_browser:
                fileheader = 'inline; filename={0}'

            filename = content_disposition_filename(filename)
            header_content = fileheader.format(filename)
            self['Content-Disposition'] = header_content
        else:
            del self['Content-Disposition']


class PDFTemplateResponse(TemplateResponse, PDFResponse):
    """Renders a Template into a PDF using electron-pdf"""

    def __init__(self, request, template, context=None, status=None,
                 content_type=None, current_app=None, filename=None,
                 show_content_in_browser=None, cmd_options=None, *args, **kwargs):

        super(PDFTemplateResponse, self).__init__(request=request,
                                                  template=template,
                                                  context=context,
                                                  status=status,
                                                  content_type=content_type,
                                                  current_app=None,
                                                  *args, **kwargs)
        self.set_filename(filename, show_content_in_browser)

        if cmd_options is None:
            cmd_options = {}
        self.cmd_options = cmd_options

    @property
    def rendered_content(self):
        """Returns the freshly rendered content for the template and context
        described by the PDFResponse.
        This *does not* set the final content of the response. To set the
        response content, you must either call render(), or set the
        content explicitly using the value of this property.
        """
        cmd_options = self.cmd_options.copy()
        return render_pdf_from_template(
            input_template=self.resolve_template(self.template_name),
            context=self.resolve_context(self.context_data),
            request=self._request,
            cmd_options=cmd_options
        )


class PDFTemplateView(TemplateView):
    """Class-based view for HTML templates rendered to PDF."""

    # Filename for downloaded PDF. If None, the response is inline.
    filename = 'rendered_pdf.pdf'

    # Send file as attachement. If True render content in the browser.
    show_content_in_browser = False

    # Filenames for the content, header, and footer templates.
    template_name = None

    # TemplateResponse classes for PDF and HTML
    response_class = PDFTemplateResponse
    html_response_class = TemplateResponse

    # Command-line options to pass to wkhtmltopdf
    cmd_options = {}

    def __init__(self, *args, **kwargs):
        super(PDFTemplateView, self).__init__(*args, **kwargs)

        # Copy self.cmd_options to prevent clobbering the class-level object.
        self.cmd_options = self.cmd_options.copy()

    def get(self, request, *args, **kwargs):
        response_class = self.response_class
        try:
            if request.GET.get('as', '') == 'html':
                # Use the html_response_class if HTML was requested.
                self.response_class = self.html_response_class
            return super(PDFTemplateView, self).get(request, *args, **kwargs)
        finally:
            # Remove self.response_class
            self.response_class = response_class

    def get_filename(self):
        return self.filename

    def get_cmd_options(self):
        return self.cmd_options

    def render_to_response(self, context, **response_kwargs):
        """
        Returns a PDF response with a template rendered with the given context.
        """
        filename = response_kwargs.pop('filename', None)
        cmd_options = response_kwargs.pop('cmd_options', None)

        if issubclass(self.response_class, PDFTemplateResponse):
            if filename is None:
                filename = self.get_filename()

            if cmd_options is None:
                cmd_options = self.get_cmd_options()

            return super(PDFTemplateView, self).render_to_response(
                context=context,
                filename=filename,
                show_content_in_browser=self.show_content_in_browser,
                cmd_options=cmd_options,
                **response_kwargs
            )
        else:
            return super(PDFTemplateView, self).render_to_response(
                context=context,
                **response_kwargs
            )



"""
Django settings for electron_pdf project.
"""

import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '%f=rv%f*qgq%k@14-l9f5si6e98pp0+p9jvsf*nfc-9q3x=7oq'

DEBUG = True


# Application definition
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'electron_pdf'
]

ROOT_URLCONF = 'electron_pdf.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


# Static files (CSS, JavaScript, Images)
STATIC_ROOT = os.path.join(BASE_DIR, 'electron_pdf', 'static')
STATIC_URL = '/static/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'electron_pdf', 'media')
MEDIA_URL = '/media/'


ELECTRON_PDF_DEBUG = False



# coding: utf-8

from distutils.core import setup

setup(
    name='django-electron-pdf',
    packages=['electron_pdf'],
    version='0.1',
    description='A Django wrapper to generate PDF from URL, HTML or Markdown files.',
    author='Madis Väin',
    author_email='madisvain@gmail.com',
    url='https://github.com/namespace-ee/django-electron-pdf',
    download_url='https://github.com/namespace-ee/django-electron-pdf/tarball/0.1',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)



# coding: utf-8

import re
import subprocess
import uuid
from tempfile import NamedTemporaryFile

try:
    from urllib.request import pathname2url
    from urllib.parse import urljoin
except ImportError:  # Python2
    from urllib import pathname2url
    from urlparse import urljoin

import django
from django.conf import settings
from django.core.files import File
from django.template import Context, RequestContext
from django.utils import six
from django.utils.encoding import smart_text


def electron_pdf(input, output_file=None, **kwargs):
    """
    Converts html to PDF using https://github.com/fraserxu/electron-pdf.
    input: input file or url of the html to be converted.
    output_file: Optional output file path. If None, the output is returned.
    **kwargs: Passed to electron-pdf via options

    example usage:
        electron_pdf(
            input='/tmp/example.html',
            landscape=True,
            printBackground=True,
            pageSize='A4'
        )
    """
    # Default options:
    options = getattr(settings, 'ELECTRON_PDF_OPTIONS', {})

    if not output_file:
        output_file = '/tmp/{}.pdf'.format(uuid.uuid4())

    subprocess.call('electron-pdf {} {}'.format(input.filename, output_file), shell=True)

    with open(output_file, 'r') as f:
        return File(f).read()


class RenderedFile(object):
    """
    Create a temporary file resource of the rendered template with context.
    The filename will be used for later conversion to PDF.
    """
    temporary_file = None
    filename = ''

    def __init__(self, template, context, request=None):
        self.temporary_file = render_to_temporary_file(
            template=template,
            context=context,
            request=request,
            prefix='electron_pdf', suffix='.html',
            delete=(not settings.ELECTRON_PDF_DEBUG)
        )
        self.filename = self.temporary_file.name

    def __del__(self):
        # Always close the temporary_file on object destruction.
        if self.temporary_file is not None:
            self.temporary_file.close()


def render_pdf_from_template(input_template, context, request=None, cmd_options={}):
    input_file = RenderedFile(
        template=input_template,
        context=context,
        request=request
    )
    return electron_pdf(input_file, **cmd_options)


def content_disposition_filename(filename):
    """
    Sanitize a file name to be used in the Content-Disposition HTTP
    header.
    Even if the standard is quite permissive in terms of
    characters, there are a lot of edge cases that are not supported by
    different browsers.
    See http://greenbytes.de/tech/tc2231/#attmultinstances for more details.
    """
    filename = filename.replace(';', '').replace('"', '')
    return http_quote(filename)


def http_quote(string):
    """
    Given a unicode string, will do its dandiest to give you back a
    valid ascii charset string you can use in, say, http headers and the
    like.
    """
    if isinstance(string, six.text_type):
        try:
            import unidecode
        except ImportError:
            pass
        else:
            string = unidecode.unidecode(string)
        string = string.encode('ascii', 'replace')
    # Wrap in double-quotes for ; , and the like
    string = string.replace(b'\\', b'\\\\').replace(b'"', b'\\"')
    return '"{0!s}"'.format(string.decode())


def pathname2fileurl(pathname):
    """Returns a file:// URL for pathname. Handles OS-specific conversions."""
    return urljoin('file:', pathname2url(pathname))


def make_absolute_paths(content):
    """Convert all MEDIA files into a file://URL paths in order to
    correctly get it displayed in PDFs."""
    overrides = [{
        'root': settings.MEDIA_ROOT,
        'url': settings.MEDIA_URL,
    }, {
        'root': settings.STATIC_ROOT,
        'url': settings.STATIC_URL,
    }]
    has_scheme = re.compile(r'^[^:/]+://')

    for x in overrides:
        if not x['url'] or has_scheme.match(x['url']):
            continue

        if not x['root'].endswith('/'):
            x['root'] += '/'

        occur_pattern = '''["|']({0}.*?)["|']'''
        occurences = re.findall(occur_pattern.format(x['url']), content)
        occurences = list(set(occurences))  # Remove dups
        for occur in occurences:
            content = content.replace(
                occur,
                pathname2fileurl(x['root']) + occur[len(x['url']):]
            )

    return content


def render_to_temporary_file(template, context, request=None, mode='w+b',
                             bufsize=-1, suffix='.html', prefix='tmp',
                             dir=None, delete=True):
    if django.VERSION < (1, 8):
        # If using a version of Django prior to 1.8, ensure ``context`` is an
        # instance of ``Context``
        if not isinstance(context, Context):
            if request:
                context = RequestContext(request, context)
            else:
                context = Context(context)
        content = template.render(context)
    else:
        content = template.render(context, request)

    content = smart_text(content)
    content = make_absolute_paths(content)

    try:
        # Python3 has 'buffering' arg instead of 'bufsize'
        tempfile = NamedTemporaryFile(
            mode=mode,
            buffering=bufsize,
            suffix=suffix,
            prefix=prefix,
            dir=dir,
            delete=delete
        )
    except TypeError:
        tempfile = NamedTemporaryFile(
            mode=mode,
            bufsize=bufsize,
            suffix=suffix,
            prefix=prefix,
            dir=dir,
            delete=delete
        )

    try:
        tempfile.write(content.encode('utf-8'))
        tempfile.flush()
        return tempfile
    except:
        # Clean-up tempfile if an Exception is raised.
        tempfile.close()
        raise
